# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ParticipantEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ParticipantEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16ParticipantEvent.proto\x12\x11opentera.protobuf\"\x92\x03\n\x10ParticipantEvent\x12\x18\n\x10participant_uuid\x18\x01 \x01(\t\x12;\n\x04type\x18\x02 \x01(\x0e\x32-.opentera.protobuf.ParticipantEvent.EventType\x12\x18\n\x10participant_name\x18\x03 \x01(\t\x12 \n\x18participant_project_name\x18\x04 \x01(\t\x12\x1d\n\x15participant_site_name\x18\x05 \x01(\t\"\xcb\x01\n\tEventType\x12\x17\n\x13PARTICIPANT_UNKNOWN\x10\x00\x12\x19\n\x15PARTICIPANT_CONNECTED\x10\x01\x12\x1c\n\x18PARTICIPANT_DISCONNECTED\x10\x02\x12\x17\n\x13PARTICIPANT_DELETED\x10\x03\x12\x15\n\x11PARTICIPANT_ADDED\x10\x04\x12\x1e\n\x1aPARTICIPANT_JOINED_SESSION\x10\x05\x12\x1c\n\x18PARTICIPANT_LEFT_SESSION\x10\x06\x62\x06proto3'
)



_PARTICIPANTEVENT_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='opentera.protobuf.ParticipantEvent.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_CONNECTED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_DISCONNECTED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_DELETED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_ADDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_JOINED_SESSION', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTICIPANT_LEFT_SESSION', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=245,
  serialized_end=448,
)
_sym_db.RegisterEnumDescriptor(_PARTICIPANTEVENT_EVENTTYPE)


_PARTICIPANTEVENT = _descriptor.Descriptor(
  name='ParticipantEvent',
  full_name='opentera.protobuf.ParticipantEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='participant_uuid', full_name='opentera.protobuf.ParticipantEvent.participant_uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='opentera.protobuf.ParticipantEvent.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='participant_name', full_name='opentera.protobuf.ParticipantEvent.participant_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='participant_project_name', full_name='opentera.protobuf.ParticipantEvent.participant_project_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='participant_site_name', full_name='opentera.protobuf.ParticipantEvent.participant_site_name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PARTICIPANTEVENT_EVENTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=448,
)

_PARTICIPANTEVENT.fields_by_name['type'].enum_type = _PARTICIPANTEVENT_EVENTTYPE
_PARTICIPANTEVENT_EVENTTYPE.containing_type = _PARTICIPANTEVENT
DESCRIPTOR.message_types_by_name['ParticipantEvent'] = _PARTICIPANTEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ParticipantEvent = _reflection.GeneratedProtocolMessageType('ParticipantEvent', (_message.Message,), {
  'DESCRIPTOR' : _PARTICIPANTEVENT,
  '__module__' : 'ParticipantEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.ParticipantEvent)
  })
_sym_db.RegisterMessage(ParticipantEvent)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: LogEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='LogEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0eLogEvent.proto\x12\x11opentera.protobuf\"\xa6\x02\n\x08LogEvent\x12\x33\n\x05level\x18\x01 \x01(\x0e\x32$.opentera.protobuf.LogEvent.LogLevel\x12\x11\n\ttimestamp\x18\x02 \x01(\x01\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x0f\n\x07message\x18\x04 \x01(\t\"\xb0\x01\n\x08LogLevel\x12\x14\n\x10LOGLEVEL_UNKNOWN\x10\x00\x12\x12\n\x0eLOGLEVEL_TRACE\x10\x07\x12\x12\n\x0eLOGLEVEL_DEBUG\x10\x06\x12\x11\n\rLOGLEVEL_INFO\x10\x05\x12\x14\n\x10LOGLEVEL_WARNING\x10\x04\x12\x15\n\x11LOGLEVEL_CRITICAL\x10\x03\x12\x12\n\x0eLOGLEVEL_ERROR\x10\x02\x12\x12\n\x0eLOGLEVEL_FATAL\x10\x01\x62\x06proto3'
)



_LOGEVENT_LOGLEVEL = _descriptor.EnumDescriptor(
  name='LogLevel',
  full_name='opentera.protobuf.LogEvent.LogLevel',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_TRACE', index=1, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_DEBUG', index=2, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_INFO', index=3, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_WARNING', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_CRITICAL', index=5, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_ERROR', index=6, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGLEVEL_FATAL', index=7, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=156,
  serialized_end=332,
)
_sym_db.RegisterEnumDescriptor(_LOGEVENT_LOGLEVEL)


_LOGEVENT = _descriptor.Descriptor(
  name='LogEvent',
  full_name='opentera.protobuf.LogEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='opentera.protobuf.LogEvent.level', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='opentera.protobuf.LogEvent.timestamp', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='opentera.protobuf.LogEvent.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='opentera.protobuf.LogEvent.message', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LOGEVENT_LOGLEVEL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=332,
)

_LOGEVENT.fields_by_name['level'].enum_type = _LOGEVENT_LOGLEVEL
_LOGEVENT_LOGLEVEL.containing_type = _LOGEVENT
DESCRIPTOR.message_types_by_name['LogEvent'] = _LOGEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogEvent = _reflection.GeneratedProtocolMessageType('LogEvent', (_message.Message,), {
  'DESCRIPTOR' : _LOGEVENT,
  '__module__' : 'LogEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.LogEvent)
  })
_sym_db.RegisterMessage(LogEvent)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TeraEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TeraEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fTeraEvent.proto\x12\x11opentera.protobuf\x1a\x19google/protobuf/any.proto\"\x9e\x01\n\tTeraEvent\x12\x33\n\x06header\x18\x01 \x01(\x0b\x32#.opentera.protobuf.TeraEvent.Header\x12$\n\x06\x65vents\x18\x02 \x03(\x0b\x32\x14.google.protobuf.Any\x1a\x36\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\x01\x12\r\n\x05topic\x18\x03 \x01(\tb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_TERAEVENT_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='opentera.protobuf.TeraEvent.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='opentera.protobuf.TeraEvent.Header.version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='opentera.protobuf.TeraEvent.Header.time', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='topic', full_name='opentera.protobuf.TeraEvent.Header.topic', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=224,
)

_TERAEVENT = _descriptor.Descriptor(
  name='TeraEvent',
  full_name='opentera.protobuf.TeraEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='opentera.protobuf.TeraEvent.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='events', full_name='opentera.protobuf.TeraEvent.events', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_TERAEVENT_HEADER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=224,
)

_TERAEVENT_HEADER.containing_type = _TERAEVENT
_TERAEVENT.fields_by_name['header'].message_type = _TERAEVENT_HEADER
_TERAEVENT.fields_by_name['events'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['TeraEvent'] = _TERAEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TeraEvent = _reflection.GeneratedProtocolMessageType('TeraEvent', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _TERAEVENT_HEADER,
    '__module__' : 'TeraEvent_pb2'
    # @@protoc_insertion_point(class_scope:opentera.protobuf.TeraEvent.Header)
    })
  ,
  'DESCRIPTOR' : _TERAEVENT,
  '__module__' : 'TeraEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.TeraEvent)
  })
_sym_db.RegisterMessage(TeraEvent)
_sym_db.RegisterMessage(TeraEvent.Header)


# @@protoc_insertion_point(module_scope)



from opentera.db.Base import db, BaseModel
from enum import Enum


class TeraSessionStatus(Enum):
    TEST_INCOMPLETE = 0
    TEST_INPROGRESS = 1
    TEST_COMPLETED = 2


class TeraTest(db.Model, BaseModel):
    __tablename__ = 't_tests'
    id_test = db.Column(db.Integer, db.Sequence('id_test_sequence'), primary_key=True, autoincrement=True)
    id_test_type = db.Column(db.Integer, db.ForeignKey('t_tests_types.id_test_type', ondelete='cascade'),
                             nullable=False)
    id_session = db.Column(db.Integer, db.ForeignKey('t_sessions.id_session', ondelete='cascade'), nullable=False)
    test_name = db.Column(db.String, nullable=False)
    test_datetime = db.Column(db.TIMESTAMP(timezone=True), nullable=False)
    test_status = db.Column(db.Integer, nullable=False)
    test_answers = db.Column(db.String, nullable=True)

    test_session = db.relationship('TeraSession')
    test_test_type = db.relationship('TeraTestType')




from opentera.db.Base import db, BaseModel


class TeraSessionParticipants(db.Model, BaseModel):
    __tablename__ = 't_sessions_participants'
    id_session_participant = db.Column(db.Integer, db.Sequence('id_session_participant'), primary_key=True,
                                       autoincrement=True)
    id_session = db.Column(db.Integer, db.ForeignKey('t_sessions.id_session'))
    id_participant = db.Column(db.Integer, db.ForeignKey('t_participants.id_participant'))

    session_participant_session = db.relationship('TeraSession')
    session_participant_participant = db.relationship('TeraParticipant')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: DeviceEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='DeviceEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11\x44\x65viceEvent.proto\x12\x11opentera.protobuf\"\xcc\x02\n\x0b\x44\x65viceEvent\x12\x13\n\x0b\x64\x65vice_uuid\x18\x01 \x01(\t\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32(.opentera.protobuf.DeviceEvent.EventType\x12\x13\n\x0b\x64\x65vice_name\x18\x03 \x01(\t\x12\x15\n\rdevice_status\x18\x04 \x01(\t\"\xc3\x01\n\tEventType\x12\x12\n\x0e\x44\x45VICE_UNKNOWN\x10\x00\x12\x14\n\x10\x44\x45VICE_CONNECTED\x10\x01\x12\x17\n\x13\x44\x45VICE_DISCONNECTED\x10\x02\x12\x12\n\x0e\x44\x45VICE_DELETED\x10\x03\x12\x10\n\x0c\x44\x45VICE_ADDED\x10\x04\x12\x19\n\x15\x44\x45VICE_JOINED_SESSION\x10\x05\x12\x17\n\x13\x44\x45VICE_LEFT_SESSION\x10\x06\x12\x19\n\x15\x44\x45VICE_STATUS_CHANGED\x10\x07\x62\x06proto3'
)



_DEVICEEVENT_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='opentera.protobuf.DeviceEvent.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEVICE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_CONNECTED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_DISCONNECTED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_DELETED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_ADDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_JOINED_SESSION', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_LEFT_SESSION', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_STATUS_CHANGED', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=178,
  serialized_end=373,
)
_sym_db.RegisterEnumDescriptor(_DEVICEEVENT_EVENTTYPE)


_DEVICEEVENT = _descriptor.Descriptor(
  name='DeviceEvent',
  full_name='opentera.protobuf.DeviceEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_uuid', full_name='opentera.protobuf.DeviceEvent.device_uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='opentera.protobuf.DeviceEvent.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='device_name', full_name='opentera.protobuf.DeviceEvent.device_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='device_status', full_name='opentera.protobuf.DeviceEvent.device_status', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DEVICEEVENT_EVENTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=373,
)

_DEVICEEVENT.fields_by_name['type'].enum_type = _DEVICEEVENT_EVENTTYPE
_DEVICEEVENT_EVENTTYPE.containing_type = _DEVICEEVENT
DESCRIPTOR.message_types_by_name['DeviceEvent'] = _DEVICEEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeviceEvent = _reflection.GeneratedProtocolMessageType('DeviceEvent', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEEVENT,
  '__module__' : 'DeviceEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.DeviceEvent)
  })
_sym_db.RegisterMessage(DeviceEvent)


# @@protoc_insertion_point(module_scope)



from opentera.db.Base import db, BaseModel


class TeraSessionTypeProject(db.Model, BaseModel):
    __tablename__ = 't_sessions_types_projects'
    id_session_type_project = db.Column(db.Integer, db.Sequence('id_session_type_project_sequence'), primary_key=True,
                                        autoincrement=True)
    id_session_type = db.Column('id_session_type', db.Integer, db.ForeignKey('t_sessions_types.id_session_type',
                                                                             ondelete='cascade'), nullable=False)
    id_project = db.Column('id_project', db.Integer, db.ForeignKey('t_projects.id_project', ondelete='cascade'),
                           nullable=False)

    session_type_project_session_type = db.relationship("TeraSessionType")
    session_type_project_project = db.relationship("TeraProject")

    def to_json(self, ignore_fields=[], minimal=False):
        ignore_fields.extend(['session_type_project_session_type', 'session_type_project_project'])

        if minimal:
            ignore_fields.extend([])

        rval = super().to_json(ignore_fields=ignore_fields)

        return rval

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraSessionType import TeraSessionType
            from opentera.db.models.TeraProject import TeraProject

            video_session = TeraSessionType.get_session_type_by_id(1)
            sensor_session = TeraSessionType.get_session_type_by_id(2)
            data_session = TeraSessionType.get_session_type_by_id(3)
            exerc_session = TeraSessionType.get_session_type_by_id(4)
            bureau_session = TeraSessionType.get_session_type_by_id(5)

            project = TeraProject.get_project_by_projectname('Default Project #1')

            stp = TeraSessionTypeProject()
            stp.session_type_project_session_type = video_session
            stp.session_type_project_project = project
            db.session.add(stp)

            stp = TeraSessionTypeProject()
            stp.session_type_project_session_type = sensor_session
            stp.session_type_project_project = project
            db.session.add(stp)

            stp = TeraSessionTypeProject()
            stp.session_type_project_session_type = data_session
            stp.session_type_project_project = project
            db.session.add(stp)

            stp = TeraSessionTypeProject()
            stp.session_type_project_session_type = exerc_session
            stp.session_type_project_project = project
            db.session.add(stp)

            stp = TeraSessionTypeProject()
            stp.session_type_project_session_type = bureau_session
            stp.session_type_project_project = project
            db.session.add(stp)

            db.session.commit()

    @staticmethod
    def get_session_type_project_by_id(stp_id: int):
        return TeraSessionTypeProject.query.filter_by(id_session_type_project=stp_id).first()

    @staticmethod
    def query_projects_for_session_type(session_type_id: int):
        return TeraSessionTypeProject.query.filter_by(id_session_type=session_type_id).all()

    @staticmethod
    def query_sessions_types_for_project(project_id: int):
        return TeraSessionTypeProject.query.filter_by(id_project=project_id).all()

    @staticmethod
    def query_session_type_project_for_session_type_project(project_id: int, session_type_id: int):
        return TeraSessionTypeProject.query.filter_by(id_project=project_id, id_session_type=session_type_id).first()

    def check_integrity(self):
        from opentera.db.models.TeraSessionType import TeraSessionType
        # If that session type is related to a service, make sure that the service is associated to that project
        if self.session_type_project_session_type.session_type_category == \
                TeraSessionType.SessionCategoryEnum.SERVICE.value:
            service_projects = [proj.id_project for proj in
                                self.session_type_project_session_type.session_type_service.service_projects]
            if self.id_project not in service_projects:
                # We must also associate that service to that project!
                from opentera.db.models.TeraServiceProject import TeraServiceProject
                new_service_project = TeraServiceProject()
                new_service_project.service_project_service = self.session_type_project_session_type \
                    .session_type_service
                new_service_project.service_project_project = self.session_type_project_project
                TeraServiceProject.insert(new_service_project)

    @classmethod
    def insert(cls, stp):
        super().insert(stp)
        stp.check_integrity()

    @classmethod
    def update(cls, update_id: int, values: dict):
        super().update(update_id, values)
        stp = TeraSessionTypeProject.get_session_type_project_by_id(update_id)
        stp.check_integrity()



from opentera.forms.TeraForm import *
from flask_babel import gettext
from modules.DatabaseModule.DBManagerTeraUserAccess import DBManagerTeraUserAccess


class TeraUserForm:

    @staticmethod
    def get_user_form(user_access: DBManagerTeraUserAccess):
        form = TeraForm("user")

        # Sections
        section = TeraFormSection("informations", gettext("Information"))
        form.add_section(section)

        # Items
        section.add_item(TeraFormItem("id_user", gettext("User ID"), "hidden", True))
        section.add_item(TeraFormItem("user_uuid", gettext("User UUID"), "hidden"))
        section.add_item(TeraFormItem("user_name", gettext("User Full Name"), "hidden"))
        section.add_item(TeraFormItem("user_username", gettext("Username"), "text", True))
        section.add_item(TeraFormItem("user_enabled", gettext("User Enabled"), "boolean", True, item_default=True))
        section.add_item(TeraFormItem("user_firstname", gettext("First Name"), "text", True))
        section.add_item(TeraFormItem("user_lastname", gettext("Last Name"), "text", True))
        section.add_item(TeraFormItem("user_email", gettext("Email"), "text"))
        section.add_item(
            TeraFormItem("user_password", gettext("Password"), "password", item_options={"confirm": True}))
        section.add_item(TeraFormItem("user_superadmin", gettext("User Is Super Administrator"), "boolean", True))
        section.add_item(TeraFormItem("user_notes", gettext("Notes"), "longtext"))
        section.add_item(TeraFormItem("user_profile", gettext("Profile"), "hidden"))
        section.add_item(TeraFormItem("user_lastonline", gettext("Last Connection"), "datetime",
                                      item_options={"readonly": True}))

        return form.to_dict()

    # @staticmethod
    # def get_user_profile_form():
    #     form = TeraForm("profile")
    #
    #     # Sections
    #     section = TeraFormSection("main_prefs", gettext("Preferences"))
    #     form.add_section(section)
    #
    #     section.add_item(TeraFormItem("language", gettext("Language"), "array", False,
    #                                   [TeraFormValue("fr", gettext("Français")),
    #                                    TeraFormValue("en", gettext("English"))]))
    #     section.add_item(TeraFormItem("notify_sounds", gettext("Activate Notification Sounds"), "boolean", False))

        # section1 = TeraFormSection("main_audio_video", gettext("Configuration audio-vidéo"))
        # form.add_section(section1)
        #
        # # Items
        # section1.add_item(TeraFormItem("camera", gettext("Caméra"), "videoinputs", True))
        # item = TeraFormItem("teracam_type", gettext("Type de caméra"), "array", True,
        #                     [TeraFormValue("0", gettext("Caméra réseau")),
        #                      TeraFormValue("1", gettext("Capture d'écran"))],
        #                     "0", TeraFormItemCondition("camera", "=", "TeraCam"))
        # section1.add_item(item)
        #
        # item = TeraFormItem("teracam_src", gettext("Adresse du flux de la caméra"), "text", True,
        #                     item_condition=TeraFormItemCondition("teracam_type", "=", 0))
        # section1.add_item(item)
        #
        # item = TeraFormItem("teracam_screen_fps", gettext("Trames par seconde"), "array", True,
        #                     ["Maximum", "5", "10", "15",
        #                      "20", "24", "30"],
        #                     item_condition=TeraFormItemCondition("teracam_type", "=", 1))
        # section1.add_item(item)
        # item = TeraFormItem("teracam_screen_res", gettext("Résolution"), "array", True,
        #                     ["Maximum", "160x120", "320x240",
        #                      "640x480", "720x480", "800x600",
        #                      "1024x768", "1280x720", "1440x900",
        #                      "1680x1050", "1920x1080"],
        #                     item_condition=TeraFormItemCondition("teracam_type", "=", 1))
        # section1.add_item(item)
        #
        # section1.add_item(TeraFormItem("camera_ptz", gettext("Caméra contrôlable (PTZ)"), "boolean"))
        # item = TeraFormItem("camera_ptz_type", gettext("Type de contrôle"), "array", True,
        #                   [TeraFormValue("0", gettext("Vivotek")), TeraFormValue("1", gettext("ONVIF (générique)"))],
        #                     item_condition=TeraFormItemCondition("camera_ptz", "=", True))
        # section1.add_item(item)
        # item = TeraFormItem("camera_ptz_ip", gettext("Adresse réseau"), "text", True,
        #                     item_condition=TeraFormItemCondition("camera_ptz", "=", True))
        # section1.add_item(item)
        # item = TeraFormItem("camera_ptz_port", gettext("Port"), "numeric", True,
        #                     item_condition=TeraFormItemCondition("camera_ptz", "=", True))
        # section1.add_item(item)
        # item = TeraFormItem("camera_ptz_username", gettext("Nom utilisateur"), "text", True,
        #                     item_condition=TeraFormItemCondition("camera_ptz", "=", True))
        # section1.add_item(item)
        # item = TeraFormItem("camera_ptz_password", gettext("Mot de passe"), "password", True,
        #                     item_condition=TeraFormItemCondition("camera_ptz", "=", True))
        # section1.add_item(item)
        #
        # section1.add_item(TeraFormItem("audio", gettext("Microphone"), "audioinputs", True))
        # section1.add_item(TeraFormItem("camera2", gettext("Caméra secondaire"), "videoinputs"))
        #
        # section2 = TeraFormSection("options", gettext("Configuration générale"))
        # form.add_section(section2)
        # section2.add_item(TeraFormItem("options_fullscreen", gettext("Affichage en plein écran en séance"), "boolean",
        #                                item_default=True))
        # section2.add_item(TeraFormItem("option_webaccess", gettext("Permettre l'accès via le web"), "boolean",
        #                                item_default=False))

        return form.to_dict()




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TeraMessage.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TeraMessage.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11TeraMessage.proto\x12\x11opentera.protobuf\x1a\x19google/protobuf/any.proto\"4\n\x0bTeraMessage\x12%\n\x07message\x18\x01 \x01(\x0b\x32\x14.google.protobuf.Anyb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_TERAMESSAGE = _descriptor.Descriptor(
  name='TeraMessage',
  full_name='opentera.protobuf.TeraMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='opentera.protobuf.TeraMessage.message', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=119,
)

_TERAMESSAGE.fields_by_name['message'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['TeraMessage'] = _TERAMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TeraMessage = _reflection.GeneratedProtocolMessageType('TeraMessage', (_message.Message,), {
  'DESCRIPTOR' : _TERAMESSAGE,
  '__module__' : 'TeraMessage_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.TeraMessage)
  })
_sym_db.RegisterMessage(TeraMessage)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: DatabaseEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='DatabaseEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13\x44\x61tabaseEvent.proto\x12\x11opentera.protobuf\"\xbe\x01\n\rDatabaseEvent\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.opentera.protobuf.DatabaseEvent.EventType\x12\x13\n\x0bobject_type\x18\x02 \x01(\t\x12\x14\n\x0cobject_value\x18\x03 \x01(\t\"H\n\tEventType\x12\x0e\n\nDB_UNKNOWN\x10\x00\x12\r\n\tDB_CREATE\x10\x01\x12\r\n\tDB_UPDATE\x10\x02\x12\r\n\tDB_DELETE\x10\x03\x62\x06proto3'
)



_DATABASEEVENT_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='opentera.protobuf.DatabaseEvent.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DB_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DB_CREATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DB_UPDATE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DB_DELETE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=161,
  serialized_end=233,
)
_sym_db.RegisterEnumDescriptor(_DATABASEEVENT_EVENTTYPE)


_DATABASEEVENT = _descriptor.Descriptor(
  name='DatabaseEvent',
  full_name='opentera.protobuf.DatabaseEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='opentera.protobuf.DatabaseEvent.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_type', full_name='opentera.protobuf.DatabaseEvent.object_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_value', full_name='opentera.protobuf.DatabaseEvent.object_value', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DATABASEEVENT_EVENTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=233,
)

_DATABASEEVENT.fields_by_name['type'].enum_type = _DATABASEEVENT_EVENTTYPE
_DATABASEEVENT_EVENTTYPE.containing_type = _DATABASEEVENT
DESCRIPTOR.message_types_by_name['DatabaseEvent'] = _DATABASEEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatabaseEvent = _reflection.GeneratedProtocolMessageType('DatabaseEvent', (_message.Message,), {
  'DESCRIPTOR' : _DATABASEEVENT,
  '__module__' : 'DatabaseEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.DatabaseEvent)
  })
_sym_db.RegisterMessage(DatabaseEvent)


# @@protoc_insertion_point(module_scope)



from opentera.db.Base import db, BaseModel
from opentera.db.models.TeraServiceRole import TeraServiceRole


class TeraServiceAccess(db.Model, BaseModel):
    __tablename__ = 't_services_access'
    id_service_access = db.Column(db.Integer, db.Sequence('id_service_project_role_sequence'), primary_key=True,
                                  autoincrement=True)
    id_user_group = db.Column(db.Integer, db.ForeignKey('t_users_groups.id_user_group', ondelete='cascade'),
                              nullable=True)
    id_device = db.Column(db.Integer, db.ForeignKey('t_devices.id_device', ondelete='cascade'), nullable=True)
    id_participant_group = db.Column(db.Integer, db.ForeignKey('t_participants_groups.id_participant_group',
                                                               ondelete='cascade'), nullable=True)
    id_service_role = db.Column(db.Integer, db.ForeignKey('t_services_roles.id_service_role', ondelete='cascade'),
                                nullable=False)

    service_access_role = db.relationship("TeraServiceRole")
    service_access_user_group = db.relationship("TeraUserGroup")
    service_access_device = db.relationship("TeraDevice")
    service_access_participant_group = db.relationship("TeraParticipantGroup")

    def __init__(self):
        pass

    def to_json(self, ignore_fields=None, minimal=False):
        if ignore_fields is None:
            ignore_fields = []

        ignore_fields.extend(['service_access_role', 'service_access_user_group',
                              'service_access_device', 'service_access_participant_group', 'id_service_role'])

        if minimal:
            ignore_fields.extend([])

        json_val = super().to_json(ignore_fields=ignore_fields)

        # Also expands with service_role infos
        if self.service_access_role:
            json_val.update(self.service_access_role.to_json(minimal=minimal))

        # Remove null values
        if not json_val['id_device']:
            del json_val['id_device']
        if not json_val['id_participant_group']:
            del json_val['id_participant_group']
        if not json_val['id_user_group']:
            del json_val['id_user_group']

        # Complete information if not minimal
        if not minimal:
            if self.service_access_role:
                json_val['service_name'] = self.service_access_role.service_role_service.service_name
            # else:
            #     # This happens on transient objects
            #     from opentera.db.models.TeraService import TeraService
            #     service = TeraService.get_service_by_id(self.id_service)
            #     if service:
            #         json_val['service_name'] = service.service_name
            if self.id_user_group:
                if self.service_access_user_group:
                    json_val['user_group_name'] = self.service_access_user_group.user_group_name
                else:
                    # This happens on transient objects
                    from opentera.db.models.TeraUserGroup import TeraUserGroup
                    ug = TeraUserGroup.get_user_group_by_id(self.id_user_group)
                    if ug:
                        json_val['user_group_name'] = ug.user_group_name
            if self.id_device:
                json_val['device_name'] = self.service_access_device.device_name
            if self.id_participant_group:
                json_val['participant_group_name'] = self.service_access_participant_group.participant_group_name
        return json_val

    @staticmethod
    def get_service_access_by_id(service_access_id: int):
        return TeraServiceAccess.query.filter_by(id_service_access=service_access_id).first()

    @staticmethod
    def update_service_access_for_user_group_for_site(id_service: int, id_user_group: int, id_service_role: int,
                                                      id_site: int):
        # Check if access already exists
        # access = TeraServiceAccess.get_specific_access_for_user_group(id_service=id_service,
        #                                                               id_user_group=id_user_group,
        #                                                               id_service_role=id_service_role)

        access = TeraServiceAccess.get_service_access_for_user_group_for_site(id_service=id_service,
                                                                              id_user_group=id_user_group,
                                                                              id_site=id_site)

        if access is None:
            # No access already present for that user - create new one
            access = TeraServiceAccess()
            access.id_user_group = id_user_group
            access.id_service_role = id_service_role
            TeraServiceAccess.insert(access)
        else:
            # Update it
            access.id_service_role = id_service_role

            db.session.commit()
        return access

    @staticmethod
    def update_service_access_for_user_group_for_project(id_service: int, id_user_group: int, id_service_role: int,
                                                         id_project: int):
        # Check if access already exists
        access = TeraServiceAccess.get_service_access_for_user_group_for_project(id_service=id_service,
                                                                                 id_user_group=id_user_group,
                                                                                 id_project=id_project)

        if access is None:
            # No access already present for that user - create new one
            access = TeraServiceAccess()
            access.id_user_group = id_user_group
            access.id_service_role = id_service_role
            TeraServiceAccess.insert(access)
        else:
            # Update it
            access.id_service_role = id_service_role

            db.session.commit()
        return access

    @staticmethod
    def get_service_access_for_user_group(id_service: int, id_user_group: int):
        return TeraServiceAccess.query.filter_by(id_user_group=id_user_group).join(TeraServiceRole)\
            .filter_by(id_service=id_service).all()

    @staticmethod
    def get_service_access_for_project(id_service: int, id_project: int):
        return TeraServiceAccess.query.join(TeraServiceRole).filter_by(id_service=id_service,
                                                                       id_project=id_project).all()

    @staticmethod
    def get_service_access_for_site(id_service: int, id_site: int):
        return TeraServiceAccess.query.join(TeraServiceRole).filter_by(id_service=id_service,
                                                                       id_site=id_site).all()

    @staticmethod
    def get_service_access_for_user_group_for_site(id_service: int, id_user_group: int, id_site: int):
        return TeraServiceAccess.query.join(TeraServiceRole).filter_by(id_service=id_service,
                                                                       id_site=id_site).filter(
            TeraServiceAccess.id_user_group == id_user_group).first()

    @staticmethod
    def get_service_access_for_user_group_for_project(id_service: int, id_user_group: int, id_project: int):
        return TeraServiceAccess.query.join(TeraServiceRole).filter_by(id_service=id_service,
                                                                       id_project=id_project).filter(
            TeraServiceAccess.id_user_group == id_user_group).first()

    @staticmethod
    def delete_service_access_for_user_group_for_site(id_site: int, id_user_group: int):
        import modules.Globals as Globals
        for service_access in TeraServiceAccess.get_service_access_for_user_group(
                id_service=Globals.opentera_service_id, id_user_group=id_user_group):
            if service_access.service_access_role.id_site == id_site:
                TeraServiceAccess.delete(service_access.id_service_access)
                break

    @staticmethod
    def delete_service_access_for_user_group_for_project(id_project: int, id_user_group: int):
        import modules.Globals as Globals
        for service_access in TeraServiceAccess.get_service_access_for_user_group(
                id_service=Globals.opentera_service_id, id_user_group=id_user_group):
            if service_access.service_access_role.id_project == id_project:
                TeraServiceAccess.delete(service_access.id_service_access)
                break

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraService import TeraService
            from opentera.db.models.TeraUserGroup import TeraUserGroup
            from opentera.db.models.TeraDevice import TeraDevice
            from opentera.db.models.TeraParticipantGroup import TeraParticipantGroup

            device = TeraDevice.get_device_by_name('Apple Watch #W05P1')
            group = TeraParticipantGroup.get_participant_group_by_group_name('Default Participant Group A')

            service_bureau = TeraService.get_service_by_key('BureauActif')
            service_bureau_admin = service_bureau.service_roles[0]
            service_bureau_user = service_bureau.service_roles[1]

            service_logging = TeraService.get_service_by_key('LoggingService')
            service_logging_admin = service_logging.service_roles[0]
            service_logging_user = service_logging.service_roles[1]

            user_group1 = TeraUserGroup.get_user_group_by_group_name('Users - Project 1')
            user_group2 = TeraUserGroup.get_user_group_by_group_name('Admins - Project 1')

            service_role = TeraServiceAccess()
            service_role.id_user_group = user_group1.id_user_group
            service_role.id_service_role = service_bureau_admin.id_service_role
            db.session.add(service_role)

            service_role = TeraServiceAccess()
            service_role.id_user_group = user_group2.id_user_group
            service_role.id_service_role = service_logging_admin.id_service_role
            db.session.add(service_role)

            service_role = TeraServiceAccess()
            service_role.id_device = device.id_device
            service_role.id_service_role = service_bureau_user.id_service_role
            db.session.add(service_role)

            service_role = TeraServiceAccess()
            service_role.id_participant_group = group.id_participant_group
            service_role.id_service_role = service_logging_user.id_service_role
            db.session.add(service_role)

            db.session.commit()



from opentera.db.Base import db, BaseModel
from opentera.db.models.TeraParticipantGroup import TeraParticipantGroup
from opentera.db.models.TeraServerSettings import TeraServerSettings

import uuid
import jwt
import time
import datetime
from passlib.hash import bcrypt


# Generator for jti
def infinite_jti_sequence():
    num = 0
    while True:
        yield num
        num += 1


# Initialize generator, call next(participant_jti_generator) to get next sequence number
participant_jti_generator = infinite_jti_sequence()


class TeraParticipant(db.Model, BaseModel):
    __tablename__ = 't_participants'
    id_participant = db.Column(db.Integer, db.Sequence('id_participant_sequence'), primary_key=True, autoincrement=True)
    participant_uuid = db.Column(db.String(36), nullable=False, unique=True)
    participant_name = db.Column(db.String, nullable=False)
    participant_username = db.Column(db.String(50), nullable=True)
    participant_email = db.Column(db.String, nullable=True)
    participant_password = db.Column(db.String, nullable=True)
    participant_token_enabled = db.Column(db.Boolean, nullable=False, default=False)
    participant_token = db.Column(db.String, nullable=True, unique=True)
    participant_lastonline = db.Column(db.TIMESTAMP(timezone=True), nullable=True)
    participant_enabled = db.Column(db.Boolean, nullable=False, default=True)
    participant_login_enabled = db.Column(db.Boolean, nullable=False, default=False)
    id_participant_group = db.Column(db.Integer, db.ForeignKey('t_participants_groups.id_participant_group',
                                                               ondelete='cascade'), nullable=True)

    id_project = db.Column(db.Integer, db.ForeignKey('t_projects.id_project', ondelete='cascade'), nullable=False)

    participant_devices = db.relationship("TeraDevice", secondary="t_devices_participants",
                                          back_populates="device_participants")

    participant_sessions = db.relationship("TeraSession", secondary="t_sessions_participants",
                                           back_populates="session_participants", passive_deletes=True)

    participant_participant_group = db.relationship("TeraParticipantGroup")

    participant_project = db.relationship("TeraProject")

    authenticated = False
    fullAccess = False

    def __init__(self):
        pass

    def __str__(self):
        return '<TeraParticipant ' + str(self.participant_name) + ' >'

    def __repr__(self):
        return self.__str__()

    def dynamic_token(self, token_key: str, expiration=3600):
        import time
        import jwt

        # Creating token with participant info
        now = time.time()
        payload = {
            'iat': int(now),
            'exp': int(now) + expiration,
            'iss': 'TeraServer',
            'jti': next(participant_jti_generator),
            'participant_uuid': self.participant_uuid,
            'id_participant': self.id_participant,
            'user_fullname': self.participant_name
        }

        return jwt.encode(payload, token_key, algorithm='HS256')

    def create_token(self):
        import random
        # Creating token with user info
        payload = {
            'iss': 'TeraServer',
            'jti': next(participant_jti_generator),
            'participant_uuid': self.participant_uuid,
            'id_participant': self.id_participant
        }

        self.participant_token = jwt.encode(payload, TeraServerSettings.get_server_setting_value(
            TeraServerSettings.ServerParticipantTokenKey), algorithm='HS256')

        return self.participant_token

    def update_last_online(self):
        self.participant_lastonline = datetime.datetime.now()
        db.session.commit()

    def to_json(self, ignore_fields=None, minimal=False):
        if ignore_fields is None:
            ignore_fields = []

        ignore_fields.extend(['authenticated', 'participant_devices',
                              'participant_sessions', 'participant_password',
                              'participant_project', 'participant_participant_group', 'fullAccess'
                              ])
        if minimal:
            ignore_fields.extend(['participant_username', 'participant_lastonline',
                                  'participant_login_enabled', 'participant_token'])

        return super().to_json(ignore_fields=ignore_fields)

    def to_json_create_event(self):
        return self.to_json(minimal=True)

    def to_json_update_event(self):
        return self.to_json(minimal=True)

    def to_json_delete_event(self):
        # Minimal information, delete can not be filtered
        return {'id_participant': self.id_participant, 'participant_uuid': self.participant_uuid}

    def is_authenticated(self):
        return self.authenticated

    def is_anonymous(self):
        return False

    def is_active(self):
        return self.participant_enabled

    def is_login_enabled(self):
        return self.participant_login_enabled

    def get_id(self):
        return self.participant_uuid

    def get_first_session(self):
        sessions = sorted(self.participant_sessions, key=lambda session: session.session_start_datetime)
        if sessions:
            return sessions[0]
        return None

    def get_last_session(self):
        sessions = sorted(self.participant_sessions, key=lambda session: session.session_start_datetime)
        if sessions:
            return sessions[-1]
        return None

    @staticmethod
    def encrypt_password(password):
        return bcrypt.hash(password)

    # @staticmethod
    # def is_anonymous():
    #     return False

    @staticmethod
    def verify_password(username, password, participant=None):
        # Query User with that username
        if participant is None:
            participant = TeraParticipant.get_participant_by_username(username)

        if participant is None:
            print('TeraParticipant: verify_password - participant ' + username + ' not found.')
            return None

        # Check if enabled
        if not participant.participant_enabled or not participant.participant_login_enabled:
            print('TeraUser: verify_password - user ' + username + ' is inactive or login is disabled.')
            return None

        # Check password
        if bcrypt.verify(password, participant.participant_password):
            participant.authenticated = True
            return participant

        return None

    @staticmethod
    def get_participant_by_token(token):
        participant = TeraParticipant.query.filter_by(participant_token=token).first()

        if participant and participant.participant_enabled and participant.participant_token_enabled:
            # Validate token
            data = jwt.decode(token.encode('utf-8'), TeraServerSettings.get_server_setting_value(
                TeraServerSettings.ServerParticipantTokenKey), algorithms='HS256')

            if data['participant_uuid'] == participant.participant_uuid:
                participant.authenticated = True
                return participant
            else:
                return None

        return None

    @staticmethod
    def get_participant_by_uuid(p_uuid):
        participant = TeraParticipant.query.filter_by(participant_uuid=p_uuid).first()

        if participant:
            return participant

        return None

    @staticmethod
    def get_participant_by_username(username):
        return TeraParticipant.query.filter_by(participant_username=username).first()

    @staticmethod
    def get_participant_by_email(email: str):
        return TeraParticipant.query.filter_by(participant_email=email).first()

    @staticmethod
    def get_participant_by_name(name):
        return TeraParticipant.query.filter_by(participant_name=name).first()

    @staticmethod
    def get_participant_by_id(part_id: int):
        return TeraParticipant.query.filter_by(id_participant=part_id).first()

    @staticmethod
    def is_participant_username_available(username: str) -> bool:
        # No username = always available
        if username is None or username == '':
            return True

        return TeraParticipant.query.filter_by(participant_username=username).first() is None

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraProject import TeraProject
            project1 = TeraProject.get_project_by_projectname('Default Project #1')

            participant1 = TeraParticipant()
            participant1.participant_name = 'Participant #1'
            participant1.participant_enabled = True
            participant1.participant_uuid = str(uuid.uuid4())
            participant1.participant_participant_group = \
                TeraParticipantGroup.get_participant_group_by_group_name('Default Participant Group A')
            participant1.participant_project = project1

            # participant1.create_token()
            participant1.participant_username = 'participant1'
            participant1.participant_password = TeraParticipant.encrypt_password('opentera')
            participant1.participant_login_enabled = True
            participant1.participant_token_enabled = True

            db.session.add(participant1)

            participant2 = TeraParticipant()
            participant2.participant_name = 'Participant #2'
            participant2.participant_enabled = False
            participant2.participant_uuid = str(uuid.uuid4())
            participant2.participant_participant_group = None
            participant2.participant_project = project1

            db.session.add(participant2)

            participant2 = TeraParticipant()
            participant2.participant_name = 'Participant #3'
            participant2.participant_enabled = True
            participant2.participant_token_enabled = True
            participant2.participant_uuid = str(uuid.uuid4())
            participant2.participant_participant_group = None
            participant2.participant_project = project1

            # participant2.create_token()
            db.session.add(participant2)

            db.session.commit()

            # Create token with added participants, since we need to have the id_participant field set
            participant1.create_token()
            participant2.create_token()
            db.session.commit()

    @classmethod
    def update(cls, update_id: int, values: dict):
        # Check if username is available
        if 'participant_username' in values:
            if not TeraParticipant.is_participant_username_available(values['participant_username']):
                raise NameError('Participant username already in use.')

        # Prevent changes on UUID
        if 'participant_uuid' in values:
            del values['participant_uuid']

        # Remove the password field is present and if empty
        if 'participant_password' in values:
            if values['participant_password'] == '':
                del values['participant_password']
            else:
                # Forcing password to string
                values['participant_password'] = TeraParticipant.encrypt_password(str(values['participant_password']))

        # Check if we need to generate or delete tokens
        if 'participant_token_enabled' in values:
            update_participant = TeraParticipant.get_participant_by_id(update_id)
            if values['participant_token_enabled'] != update_participant.participant_token_enabled:
                if 'participant_enabled' in values:
                    participant_enabled = values['participant_enabled']
                else:
                    participant_enabled = update_participant.participant_enabled
                # Value changed
                if not values['participant_token_enabled'] or not participant_enabled:
                    values['participant_token'] = None  # Remove token
                else:
                    values['participant_token'] = update_participant.create_token()  # Generate new token
                    db.session.rollback()  # Don't save token here

        super().update(update_id, values)

    @classmethod
    def insert(cls, participant):
        # Encrypts password
        if participant.participant_password:
            # Forcing password to string
            participant.participant_password = TeraParticipant.encrypt_password(str(participant.participant_password))

        participant.participant_lastonline = None
        participant.participant_uuid = str(uuid.uuid4())
        participant.participant_token = None
        # participant.create_token()
        # Check if username is available
        if not TeraParticipant.is_participant_username_available(participant.participant_username):
            raise NameError('Participant username already in use.')
        super().insert(participant)

        # Token must be created after being inserted, since we need to have a valid ID participant into it
        if participant.participant_token_enabled and participant.participant_enabled:
            participant.create_token()
        db.session.commit()

    @classmethod
    def delete(cls, id_todel: int):
        super().delete(id_todel)
        # Check if we need to delete orphan sessions (sessions that have no more participants left
        # from opentera.db.models.TeraSession import TeraSession
        # TeraSession.delete_orphaned_sessions(False)
        #
        # db.session.commit()



# from opentera.db.Base import db, BaseModel
# import uuid
# import datetime
# import os
# from shutil import rmtree
# from modules.FlaskModule.FlaskModule import flask_app
#
#
# class TeraDeviceData(db.Model, BaseModel):
#     __tablename__ = "t_devices_data"
#     id_device_data = db.Column(db.Integer, db.Sequence("id_device_data_sequence"), primary_key=True, autoincrement=True)
#     id_device = db.Column(db.Integer, db.ForeignKey("t_devices.id_device", ondelete='cascade'), nullable=False)
#     id_session = db.Column(db.Integer, db.ForeignKey("t_sessions.id_session", ondelete='cascade'), nullable=False)
#     devicedata_name = db.Column(db.String, nullable=True)
#     devicedata_original_filename = db.Column(db.String, nullable=False)
#     devicedata_saved_date = db.Column(db.TIMESTAMP, nullable=False)
#     devicedata_uuid = db.Column(db.String(36), nullable=False, unique=True)
#     devicedata_filesize = db.Column(db.Integer, nullable=False)
#
#     devicedata_device = db.relationship("TeraDevice")
#     devicedata_session = db.relationship("TeraSession")
#
#     def to_json(self, ignore_fields=None, minimal=False):
#         if ignore_fields is None:
#             ignore_fields = []
#
#         ignore_fields.extend(['devicedata_device', 'devicedata_session', 'devicedata_uuid'])
#         rval = super().to_json(ignore_fields=ignore_fields)
#
#         return rval
#
#     @staticmethod
#     def get_data_for_device(device_id: int):
#         return TeraDeviceData.query.filter_by(id_device=device_id).all()
#
#     @staticmethod
#     def get_data_by_id(device_data_id: int):
#         return TeraDeviceData.query.filter_by(id_device_data=device_data_id).first()
#
#     @staticmethod
#     def get_data_by_uuid(device_data_uuid):
#         return TeraDeviceData.query.filter_by(devicedata_uuid=device_data_uuid).first()
#
#     @staticmethod
#     def get_data_for_session(session_id: int):
#         return TeraDeviceData.query.filter_by(id_session=session_id).all()
#
#     @staticmethod
#     def get_data_for_participant(part_id: int):
#         from opentera.db.models.TeraSession import TeraSession
#         from opentera.db.models.TeraParticipant import TeraParticipant
#         return TeraDeviceData.query.join(TeraSession).filter(TeraSession.session_participants.
#                                                              any(id_participant=part_id)).all()
#
#     @staticmethod
#     def create_defaults(upload_path: str):
#         from .TeraSession import TeraSession
#         from .TeraDevice import TeraDevice
#         import os
#
#         # Clear all files in upload path
#         if os.path.exists(upload_path):
#             rmtree(upload_path)
#
#         if not os.path.exists(upload_path):
#              os.mkdir(upload_path)
#
#         base_session = TeraSession.get_session_by_name('Séance #1')
#         base_device = TeraDevice.get_device_by_name('Apple Watch #W05P1')
#         data1 = TeraDeviceData()
#         data1.id_device = base_device.id_device
#         data1.id_session = base_session.id_session
#         data1.devicedata_name = 'Test Data #1'
#         data1.devicedata_original_filename = 'Data1.data'
#         data1.devicedata_saved_date = datetime.datetime.now()
#         data1.devicedata_uuid = str(uuid.uuid4())
#         # Create "file"
#         filesize = 1024 * 1024
#         with open(upload_path + '/' + data1.devicedata_uuid, 'wb') as fout:
#             fout.write(os.urandom(filesize))
#         data1.devicedata_filesize = filesize
#         db.session.add(data1)
#
#         data2 = TeraDeviceData()
#         data2.id_device = base_device.id_device
#         data2.id_session = base_session.id_session
#         data2.devicedata_name = 'Test Data #2'
#         data2.devicedata_original_filename = 'Data2.data'
#         data2.devicedata_saved_date = datetime.datetime.now()
#         data2.devicedata_uuid = str(uuid.uuid4())
#         # Create "file"
#         filesize = 1024 * 1024 * 10
#         with open(upload_path + '/' + data2.devicedata_uuid, 'wb') as fout:
#             fout.write(os.urandom(filesize))
#         data2.devicedata_filesize = filesize
#         db.session.add(data2)
#         db.session.commit()
#
#     def delete(self):
#         # file_path = flask_app.config['UPLOAD_FOLDER']
#         # Delete physical file from the disk
#         # os.remove(os.path.join(file_path, self.devicedata_uuid))
#         TeraDeviceData.delete_files([self])
#
#         # Delete data from the database
#         db.session.delete(self)
#         db.session.commit()
#
#     @staticmethod
#     def delete_files(datas: list):
#         file_path = flask_app.config['UPLOAD_FOLDER']
#
#         for data in datas:
#             file_name = os.path.join(file_path, data.devicedata_uuid)
#             if os.path.exists(file_name):
#                 print('TeraDeviceData: Deleted ' + file_name)
#                 os.remove(file_name)
#             else:
#                 print('TeraDeviceData: File not found: ' + file_name)
#
#     @staticmethod
#     def delete_files_for_device(id_device: int):
#         datas = TeraDeviceData.get_data_for_device(id_device)
#         TeraDeviceData.delete_files(datas)
#
#     @staticmethod
#     def delete_files_for_session(id_session: int):
#         datas = TeraDeviceData.get_data_for_session(id_session)
#         TeraDeviceData.delete_files(datas)
#
#     @staticmethod
#     def delete_orphaned_files():
#         # Use with caution - this will take some time to process with a lot of files...
#         file_path = flask_app.config['UPLOAD_FOLDER']
#         for file in os.listdir(file_path):
#             if not TeraDeviceData.get_data_by_uuid(file):
#                 file_name = os.path.join(file_path, file)
#                 print('TeraDeviceData: Orphaned file found and deleted: ' + file_name)
#                 os.remove(file_name)
#



from opentera.db.Base import db, BaseModel


tests_types_projects_table = db.Table('t_tests_types_projects', db.Column('id_tests_types', db.Integer, db.ForeignKey(
                                                                                  't_tests_types.id_test_type',
                                                                                  ondelete='cascade')),
                                      db.Column('id_project', db.Integer, db.ForeignKey('t_projects.id_project',
                                                                                        ondelete='cascade')))


class TeraTestType(db.Model, BaseModel):
    __tablename__ = 't_tests_types'
    id_test_type = db.Column(db.Integer, db.Sequence('id_test_type_sequence'), primary_key=True, autoincrement=True)
    test_type_name = db.Column(db.String, nullable=False, unique=False)
    test_type_prefix = db.Column(db.String(10), nullable=False, unique=False)
    test_type_definition = db.Column(db.String, nullable=False)

    test_type_projects = db.relationship("TeraProject", secondary=tests_types_projects_table)



from opentera.db.Base import db, BaseModel


class TeraUserGroup(db.Model, BaseModel):
    __tablename__ = 't_users_groups'
    id_user_group = db.Column(db.Integer, db.Sequence('id_usergroup_sequence'), primary_key=True, autoincrement=True)
    user_group_name = db.Column(db.String, nullable=False, unique=False)

    user_group_services_access = db.relationship('TeraServiceAccess', cascade="all,delete")
    user_group_users = db.relationship("TeraUser", secondary="t_users_users_groups", back_populates="user_user_groups",
                                       passive_deletes=True)

    def to_json(self, ignore_fields=None, minimal=False):
        if ignore_fields is None:
            ignore_fields = []
        ignore_fields.extend(['user_group_users', 'user_group_services_access', 'user_group_projects_access'])
        if minimal:
            ignore_fields.extend([])
        rval = super().to_json(ignore_fields=ignore_fields)

        return rval

    def to_json_create_event(self):
        return self.to_json(minimal=True)

    def to_json_update_event(self):
        return self.to_json(minimal=True)

    def to_json_delete_event(self):
        # Minimal information, delete can not be filtered
        return {'id_user_group': self.id_user_group}

    def get_projects_roles(self, no_inheritance: bool = False) -> dict:
        projects_roles = {}

        # Projects
        for service_access in self.user_group_services_access:
            if service_access.service_access_role.id_project:
                projects_roles[service_access.service_access_role.service_role_project] = \
                    {'project_role': service_access.service_access_role.service_role_name, 'inherited': False}

        # Sites - if we are admin in a site, we are automatically admin in all its project
        if not no_inheritance:
            for service_access in self.user_group_services_access:
                if service_access.service_access_role.id_site:
                    if service_access.service_access_role.service_role_name == 'admin':
                        for project in service_access.service_access_role.service_role_site.site_projects:
                            projects_roles[project] = {'project_role': 'admin', 'inherited': True}

        return projects_roles

    def get_sites_roles(self) -> dict:
        sites_roles = {}
        # Sites
        for service_access in self.user_group_services_access:
            if service_access.service_access_role.id_site:
                sites_roles[service_access.service_access_role.service_role_site] = \
                    {'site_role': service_access.service_access_role.service_role_name, 'inherited': False}

        # Projects - each project's site also provides a "user" access for that site
        for service_access in self.user_group_services_access:
            if service_access.service_access_role.id_project:
                project_site = service_access.service_access_role.service_role_project.project_site
                if project_site not in sites_roles:
                    sites_roles[project_site] = {'site_role': 'user', 'inherited': True}

        return sites_roles

    @staticmethod
    def get_user_group_by_group_name(name: str):
        return TeraUserGroup.query.filter_by(user_group_name=name).first()

    @staticmethod
    def get_user_group_by_id(group_id: int):
        return TeraUserGroup.query.filter_by(id_user_group=group_id).first()

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraProject import TeraProject
            from opentera.db.models.TeraServiceAccess import TeraServiceAccess
            from opentera.db.models.TeraServiceRole import TeraServiceRole
            from opentera.db.models.TeraService import TeraService
            from opentera.db.models.TeraSite import TeraSite

            opentera_service_id = TeraService.get_openteraserver_service().id_service

            ugroup = TeraUserGroup()
            ugroup.user_group_name = "Users - Projects 1 & 2"
            db.session.add(ugroup)

            ugroup = TeraUserGroup()
            ugroup.user_group_name = "Admins - Project 1"
            db.session.add(ugroup)

            ugroup = TeraUserGroup()
            ugroup.user_group_name = "Admins - Default Site"
            db.session.add(ugroup)

            ugroup = TeraUserGroup()
            ugroup.user_group_name = "Users - Project 1"
            db.session.add(ugroup)

            ugroup = TeraUserGroup()
            ugroup.user_group_name = "No access!"
            db.session.add(ugroup)

            db.session.commit()

            id_user_group = TeraUserGroup.get_user_group_by_group_name('Users - Projects 1 & 2').id_user_group
            access = TeraServiceAccess()
            access.id_user_group = id_user_group
            id_project = TeraProject.get_project_by_projectname('Default Project #1').id_project
            user_role = TeraServiceRole.get_specific_service_role_for_project(service_id=opentera_service_id,
                                                                              project_id=id_project, rolename='user')
            access.id_service_role = user_role.id_service_role
            db.session.add(access)

            access = TeraServiceAccess()
            access.id_user_group = id_user_group
            id_project = TeraProject.get_project_by_projectname('Default Project #2').id_project
            user_role = TeraServiceRole.get_specific_service_role_for_project(service_id=opentera_service_id,
                                                                              project_id=id_project, rolename='user')
            access.id_service_role = user_role.id_service_role
            db.session.add(access)

            admin_access = TeraServiceAccess()
            admin_role = TeraServiceRole.get_specific_service_role_for_site(service_id=opentera_service_id,
                                                                            site_id=TeraSite
                                                                            .get_site_by_sitename('Default Site').id_site,
                                                                            rolename='admin')
            admin_access.id_service_role = admin_role.id_service_role
            admin_access.id_user_group = TeraUserGroup.get_user_group_by_group_name('Admins - Default Site').id_user_group
            db.session.add(admin_access)

            access = TeraServiceAccess()
            access.id_user_group = TeraUserGroup.get_user_group_by_group_name('Admins - Project 1').id_user_group
            id_project = TeraProject.get_project_by_projectname('Default Project #1').id_project
            admin_role = TeraServiceRole.get_specific_service_role_for_project(service_id=opentera_service_id,
                                                                               project_id=id_project, rolename='admin')
            access.id_service_role = admin_role.id_service_role
            db.session.add(access)

            access = TeraServiceAccess()
            access.id_user_group = TeraUserGroup.get_user_group_by_group_name('Users - Project 1').id_user_group
            user_role = TeraServiceRole.get_specific_service_role_for_project(service_id=opentera_service_id,
                                                                              project_id=id_project, rolename='user')
            access.id_service_role = user_role.id_service_role
            db.session.add(access)

            db.session.commit()




from opentera.db.Base import db, BaseModel

from enum import Enum
import random
from datetime import datetime, timedelta
import uuid


class TeraSessionStatus(Enum):
    STATUS_NOTSTARTED = 0
    STATUS_INPROGRESS = 1
    STATUS_COMPLETED = 2
    STATUS_CANCELLED = 3
    STATUS_TERMINATED = 4


class TeraSession(db.Model, BaseModel):
    __tablename__ = 't_sessions'
    id_session = db.Column(db.Integer, db.Sequence('id_session_sequence'), primary_key=True, autoincrement=True)
    session_uuid = db.Column(db.String(36), nullable=False, unique=True)

    id_session_type = db.Column(db.Integer, db.ForeignKey('t_sessions_types.id_session_type'), nullable=False)
    id_creator_user = db.Column(db.Integer, db.ForeignKey('t_users.id_user'), nullable=True)
    id_creator_device = db.Column(db.Integer, db.ForeignKey('t_devices.id_device'), nullable=True)
    id_creator_participant = db.Column(db.Integer, db.ForeignKey('t_participants.id_participant'), nullable=True)
    id_creator_service = db.Column(db.Integer, db.ForeignKey('t_services.id_service', ondelete='set null'),
                                   nullable=True)

    session_name = db.Column(db.String, nullable=False)
    session_start_datetime = db.Column(db.TIMESTAMP(timezone=True), nullable=False)
    session_duration = db.Column(db.Integer, nullable=False, default=0)
    session_status = db.Column(db.Integer, nullable=False)
    session_comments = db.Column(db.String, nullable=True)
    session_parameters = db.Column(db.String, nullable=True)

    session_participants = db.relationship("TeraParticipant", secondary="t_sessions_participants",
                                           back_populates="participant_sessions")
    session_users = db.relationship("TeraUser", secondary="t_sessions_users", back_populates="user_sessions")
    session_devices = db.relationship("TeraDevice", secondary="t_sessions_devices",
                                      back_populates="device_sessions")

    session_creator_user = db.relationship('TeraUser')
    session_creator_device = db.relationship('TeraDevice')
    session_creator_participant = db.relationship('TeraParticipant')
    session_creator_service = db.relationship('TeraService')

    session_session_type = db.relationship('TeraSessionType')
    session_events = db.relationship('TeraSessionEvent', cascade="delete")
    session_assets = db.relationship('TeraAsset', cascade='delete')

    def to_json(self, ignore_fields=None, minimal=False):
        if ignore_fields is None:
            ignore_fields = []

        ignore_fields.extend(['session_participants', 'session_creator_user', 'session_creator_device',
                              'session_creator_participant', 'session_creator_service', 'session_session_type',
                              'session_events', 'session_users', 'session_devices', 'session_assets'])
        if minimal:
            ignore_fields.extend(['session_comments', 'session_duration', 'session_start_datetime',
                                  'session_parameters'])

        rval = super().to_json(ignore_fields=ignore_fields)

        if not minimal:
            # Append list of participants ids and names
            rval['session_participants'] = [{'id_participant': part.id_participant,
                                             'participant_uuid': part.participant_uuid,
                                             'participant_name': part.participant_name,
                                             'id_project': part.id_project}
                                            for part in self.session_participants]

            # Append list of users ids and names
            rval['session_users'] = [{'id_user': user.id_user,
                                      'user_uuid': user.user_uuid,
                                      'user_name': user.get_fullname()}
                                     for user in self.session_users]

            # Append list of devices ids and names
            rval['session_devices'] = [{'id_device': device.id_device,
                                        'device_uuid': device.device_uuid,
                                        'device_name': device.device_name}
                                       for device in self.session_devices]

            # Append user name
            if self.session_creator_user:
                rval['session_creator_user'] = self.session_creator_user.get_fullname()
                rval['session_creator_user_uuid'] = self.session_creator_user.user_uuid
            elif self.session_creator_device:
                rval['session_creator_device'] = self.session_creator_device.device_name
                rval['session_creator_device_uuid'] = self.session_creator_device.device_uuid
            elif self.session_creator_participant:
                rval['session_creator_participant'] = self.session_creator_participant.participant_name
                rval['session_creator_participant_uuid'] = self.session_creator_participant.participant_uuid
            elif self.session_creator_service:
                rval['session_creator_service'] = self.session_creator_service.service_name
                rval['session_creator_service_uuid'] = self.session_creator_service.service_uuid

            # Append session components
            # from opentera.db.models.TeraDeviceData import TeraDeviceData
            # rval['session_has_device_data'] = len(TeraDeviceData.get_data_for_session(self.id_session)) > 0
        return rval

    def to_json_create_event(self):
        return self.to_json(minimal=True)

    def to_json_update_event(self):
        return self.to_json(minimal=True)

    def to_json_delete_event(self):
        # Minimal information, delete can not be filtered
        return {'id_session': self.id_session, 'session_uuid': self.session_uuid}

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraUser import TeraUser
            from opentera.db.models.TeraDevice import TeraDevice
            from opentera.db.models.TeraSessionType import TeraSessionType
            from opentera.db.models.TeraParticipant import TeraParticipant
            from opentera.db.models.TeraService import TeraService

            session_user = TeraUser.get_user_by_id(1)
            session_user2 = TeraUser.get_user_by_id(2)
            session_part = TeraParticipant.get_participant_by_name('Participant #1')
            session_part2 = TeraParticipant.get_participant_by_name('Participant #2')
            session_service = TeraService.get_service_by_key('VideoRehabService')
            session_device = TeraDevice.get_device_by_id(2)

            # Create user sessions
            for i in range(8):
                base_session = TeraSession()
                base_session.session_creator_user = session_user
                ses_type = random.randint(1, 4)
                base_session.session_session_type = TeraSessionType.get_session_type_by_id(ses_type)
                base_session.session_name = "Séance #" + str(i + 1)
                base_session.session_start_datetime = datetime.now() - timedelta(days=random.randint(0, 30))
                base_session.session_duration = random.randint(60, 4800)
                ses_status = random.randint(0, 4)
                base_session.session_status = ses_status
                if i < 7:
                    base_session.session_participants = [session_part]
                else:
                    base_session.session_participants = [session_part, session_part2]
                if i < 4:
                    base_session.session_users = [base_session.session_creator_user]
                else:
                    base_session.session_users = [base_session.session_creator_user, session_user2]
                if i == 3:
                    base_session.session_devices = [session_device]
                base_session.session_uuid = str(uuid.uuid4())
                db.session.add(base_session)

            # Create device sessions
            for i in range(8):
                base_session = TeraSession()
                base_session.session_creator_device = TeraDevice.get_device_by_id(1)
                ses_type = random.randint(1, 4)
                base_session.session_session_type = TeraSessionType.get_session_type_by_id(ses_type)
                base_session.session_name = "Séance #" + str(i + 1)
                base_session.session_start_datetime = datetime.now() - timedelta(days=random.randint(0, 30))
                base_session.session_duration = random.randint(60, 4800)
                ses_status = random.randint(0, 4)
                base_session.session_status = ses_status
                if i < 7:
                    base_session.session_participants = [session_part]
                else:
                    base_session.session_participants = [session_part, session_part2]
                base_session.session_uuid = str(uuid.uuid4())
                db.session.add(base_session)

            # Create participant sessions
            for i in range(8):
                base_session = TeraSession()
                base_session.session_creator_participant = TeraParticipant.get_participant_by_id(1)
                ses_type = random.randint(1, 4)
                base_session.session_session_type = TeraSessionType.get_session_type_by_id(ses_type)
                base_session.session_name = "Séance #" + str(i + 1)
                base_session.session_start_datetime = datetime.now() - timedelta(days=random.randint(0, 30))
                base_session.session_duration = random.randint(60, 4800)
                ses_status = random.randint(0, 4)
                base_session.session_status = ses_status
                base_session.session_participants = [base_session.session_creator_participant]
                base_session.session_uuid = str(uuid.uuid4())
                db.session.add(base_session)

            # Create service sessions
            for i in range(4):
                base_session = TeraSession()
                base_session.session_creator_service = session_service
                ses_type = random.randint(1, 4)
                base_session.session_session_type = TeraSessionType.get_session_type_by_id(ses_type)
                base_session.session_name = "Séance #" + str(i + 1)
                base_session.session_start_datetime = datetime.now() - timedelta(days=random.randint(0, 30))
                base_session.session_duration = random.randint(60, 4800)
                ses_status = random.randint(0, 4)
                base_session.session_status = ses_status
                if i < 3:
                    base_session.session_participants = [session_part]
                else:
                    base_session.session_participants = [session_part, session_part2]
                base_session.session_uuid = str(uuid.uuid4())
                db.session.add(base_session)

            db.session.commit()

    @staticmethod
    def get_session_by_id(ses_id: int):
        return TeraSession.query.filter_by(id_session=ses_id).first()

    @staticmethod
    def get_session_by_uuid(s_uuid):
        session = TeraSession.query.filter_by(session_uuid=s_uuid).first()
        if session:
            return session

        return None

    @staticmethod
    def get_session_by_name(name: str):
        return TeraSession.query.filter_by(session_name=name).first()

    @staticmethod
    def get_sessions_for_participant(part_id: int):
        from opentera.db.models.TeraParticipant import TeraParticipant
        return TeraSession.query.join(TeraSession.session_participants).filter(TeraParticipant.id_participant ==
                                                                               part_id) \
            .order_by(TeraSession.session_start_datetime.desc()).all()

    @staticmethod
    def get_sessions_for_user(user_id: int):
        from opentera.db.models.TeraUser import TeraUser
        return TeraSession.query.join(TeraSession.session_users).filter(TeraUser.id_user == user_id) \
            .order_by(TeraSession.session_start_datetime.desc()).all()

    @staticmethod
    def get_sessions_for_device(device_id: int):
        from opentera.db.models.TeraDevice import TeraDevice
        return TeraSession.query.join(TeraSession.session_devices).filter(TeraDevice.id_device == device_id) \
            .order_by(TeraSession.session_start_datetime.desc()).all()

    @staticmethod
    def get_sessions_for_type(session_type_id: int):
        return TeraSession.query.filter_by(id_session_type=session_type_id).all()

    @staticmethod
    def is_user_in_session(session_uuid: str, user_uuid: str) -> bool:
        session = TeraSession.get_session_by_uuid(session_uuid)
        user_uuids = [user.user_uuid for user in session.session_users]
        return user_uuid in user_uuids

    @staticmethod
    def is_device_in_session(session_uuid: str, device_uuid: str) -> bool:
        session = TeraSession.get_session_by_uuid(session_uuid)
        device_uuids = [device.device_uuid for device in session.session_devices]
        return device_uuid in device_uuids

    @staticmethod
    def is_participant_in_session(session_uuid: str, participant_uuid: str) -> bool:
        session = TeraSession.get_session_by_uuid(session_uuid)
        participant_uuids = [participant.participant_uuid for participant in session.session_participants]
        return participant_uuid in participant_uuids

    def has_user(self, id_user: int) -> bool:
        user_ids = [user.id_user for user in self.session_users]
        return id_user in user_ids

    def has_device(self, id_device: int) -> bool:
        device_ids = [device.id_device for device in self.session_devices]
        return id_device in device_ids

    def has_participant(self, id_participant: int) -> bool:
        participant_ids = [participant.id_participant for participant in self.session_participants]
        return id_participant in participant_ids

    def get_associated_project_id(self):
        project_id = None
        if self.session_participants:
            # Return project id for the first participant, since they should all be the same...
            project_id = self.session_participants[0].id_project

        return project_id

    # THIS SHOULD NOT BE USED ANYMORE, AS DELETES CAN'T OCCUR IF THERE'S STILL ASSOCIATED SESSIONS
    # @staticmethod
    # def delete_orphaned_sessions(commit_changes=True):
    #     from opentera.db.models.TeraDeviceData import TeraDeviceData
    #     orphans_parts = TeraSession.query.outerjoin(TeraSession.session_participants).filter(
    #         TeraSession.session_participants == None).all()
    #
    #     orphans_users = TeraSession.query.outerjoin(TeraSession.session_users).filter(
    #         TeraSession.session_users == None).all()
    #
    #     orphans = list(set(orphans_parts + orphans_users))  # Keep unique sessions only!
    #
    #     if orphans:
    #         for orphan in orphans:
    #             TeraDeviceData.delete_files_for_session(orphan.id_session)
    #             db.session.delete(orphan)
    #             # TeraSession.delete(orphan.id_session)
    #
    #     if commit_changes:
    #         db.session.commit()

    @classmethod
    def delete(cls, id_todel):
        # from opentera.db.models.TeraDeviceData import TeraDeviceData
        # TeraDeviceData.delete_files_for_session(id_todel)
        super().delete(id_todel)

    @classmethod
    def insert(cls, session):
        session.session_uuid = str(uuid.uuid4())
        super().insert(session)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: UserRegisterToEvent.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='UserRegisterToEvent.proto',
  package='opentera.protobuf',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x19UserRegisterToEvent.proto\x12\x11opentera.protobuf\"\xa9\x05\n\x13UserRegisterToEvent\x12\x44\n\nevent_type\x18\x01 \x01(\x0e\x32\x30.opentera.protobuf.UserRegisterToEvent.EventType\x12=\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32-.opentera.protobuf.UserRegisterToEvent.Action\"\xc2\x03\n\tEventType\x12\x11\n\rEVENT_UNKNOWN\x10\x00\x12\x18\n\x14\x45VENT_USER_CONNECTED\x10\x01\x12\x1b\n\x17\x45VENT_USER_DISCONNECTED\x10\x02\x12\x16\n\x12\x45VENT_USER_DELETED\x10\x03\x12\x14\n\x10\x45VENT_USER_ADDED\x10\x04\x12\x16\n\x12\x45VENT_USER_UPDATED\x10\x05\x12\x1f\n\x1b\x45VENT_PARTICIPANT_CONNECTED\x10\x06\x12\"\n\x1e\x45VENT_PARTICIPANT_DISCONNECTED\x10\x07\x12\x1d\n\x19\x45VENT_PARTICIPANT_DELETED\x10\x08\x12\x1b\n\x17\x45VENT_PARTICIPANT_ADDED\x10\t\x12\x1d\n\x19\x45VENT_PARTICIPANT_UPDATED\x10\n\x12\x1a\n\x16\x45VENT_DEVICE_CONNECTED\x10\x0b\x12\x1d\n\x19\x45VENT_DEVICE_DISCONNECTED\x10\x0c\x12\x18\n\x14\x45VENT_DEVICE_DELETED\x10\r\x12\x16\n\x12\x45VENT_DEVICE_ADDED\x10\x0e\x12\x18\n\x14\x45VENT_DEVICE_UPDATED\x10\x0f\"H\n\x06\x41\x63tion\x12\x12\n\x0e\x41\x43TION_UNKNOWN\x10\x00\x12\x13\n\x0f\x41\x43TION_REGISTER\x10\x01\x12\x15\n\x11\x41\x43TION_UNREGISTER\x10\x02\x62\x06proto3'
)



_USERREGISTERTOEVENT_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='opentera.protobuf.UserRegisterToEvent.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EVENT_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_USER_CONNECTED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_USER_DISCONNECTED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_USER_DELETED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_USER_ADDED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_USER_UPDATED', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_PARTICIPANT_CONNECTED', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_PARTICIPANT_DISCONNECTED', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_PARTICIPANT_DELETED', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_PARTICIPANT_ADDED', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_PARTICIPANT_UPDATED', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_DEVICE_CONNECTED', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_DEVICE_DISCONNECTED', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_DEVICE_DELETED', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_DEVICE_ADDED', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_DEVICE_UPDATED', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=206,
  serialized_end=656,
)
_sym_db.RegisterEnumDescriptor(_USERREGISTERTOEVENT_EVENTTYPE)

_USERREGISTERTOEVENT_ACTION = _descriptor.EnumDescriptor(
  name='Action',
  full_name='opentera.protobuf.UserRegisterToEvent.Action',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ACTION_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACTION_REGISTER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACTION_UNREGISTER', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=658,
  serialized_end=730,
)
_sym_db.RegisterEnumDescriptor(_USERREGISTERTOEVENT_ACTION)


_USERREGISTERTOEVENT = _descriptor.Descriptor(
  name='UserRegisterToEvent',
  full_name='opentera.protobuf.UserRegisterToEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='opentera.protobuf.UserRegisterToEvent.event_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action', full_name='opentera.protobuf.UserRegisterToEvent.action', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _USERREGISTERTOEVENT_EVENTTYPE,
    _USERREGISTERTOEVENT_ACTION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=730,
)

_USERREGISTERTOEVENT.fields_by_name['event_type'].enum_type = _USERREGISTERTOEVENT_EVENTTYPE
_USERREGISTERTOEVENT.fields_by_name['action'].enum_type = _USERREGISTERTOEVENT_ACTION
_USERREGISTERTOEVENT_EVENTTYPE.containing_type = _USERREGISTERTOEVENT
_USERREGISTERTOEVENT_ACTION.containing_type = _USERREGISTERTOEVENT
DESCRIPTOR.message_types_by_name['UserRegisterToEvent'] = _USERREGISTERTOEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UserRegisterToEvent = _reflection.GeneratedProtocolMessageType('UserRegisterToEvent', (_message.Message,), {
  'DESCRIPTOR' : _USERREGISTERTOEVENT,
  '__module__' : 'UserRegisterToEvent_pb2'
  # @@protoc_insertion_point(class_scope:opentera.protobuf.UserRegisterToEvent)
  })
_sym_db.RegisterMessage(UserRegisterToEvent)


# @@protoc_insertion_point(module_scope)



from opentera.db.Base import db, BaseModel


class TeraProject(db.Model, BaseModel):
    __tablename__ = 't_projects'
    id_project = db.Column(db.Integer, db.Sequence('id_project_sequence'), primary_key=True, autoincrement=True)
    id_site = db.Column(db.Integer, db.ForeignKey('t_sites.id_site', ondelete='cascade'), nullable=False)
    project_name = db.Column(db.String, nullable=False, unique=False)

    project_site = db.relationship("TeraSite")
    project_participants = db.relationship("TeraParticipant", passive_deletes=True)
    project_participants_groups = db.relationship("TeraParticipantGroup", passive_deletes=True)
    project_devices = db.relationship("TeraDevice", secondary="t_devices_projects", back_populates="device_projects")
    project_session_types = db.relationship("TeraSessionType", secondary="t_sessions_types_projects",
                                            back_populates="session_type_projects")

    def to_json(self, ignore_fields=None, minimal=False):
        if ignore_fields is None:
            ignore_fields = []

        ignore_fields.extend(['project_site', 'project_participants', 'project_participants_groups', 'project_devices',
                              'project_session_types'])
        rval = super().to_json(ignore_fields=ignore_fields)

        # Add sitename
        if 'site_name' not in ignore_fields and not minimal:
            rval['site_name'] = self.project_site.site_name

        return rval

    def to_json_create_event(self):
        return self.to_json(minimal=True)

    def to_json_update_event(self):
        return self.to_json(minimal=True)

    def to_json_delete_event(self):
        # Minimal information, delete can not be filtered
        return {'id_project': self.id_project}

    def get_users_ids_in_project(self):
        # Get all users who has a role in the project
        users = self.get_users_in_project()
        users_ids = []

        for user in users:
            users_ids.append(user.id_user)

        return users_ids

    def get_users_in_project(self):
        import modules.Globals as Globals
        from opentera.db.models.TeraServiceAccess import TeraServiceAccess
        from opentera.db.models.TeraUser import TeraUser
        # Get all users who has a role in the project
        project_access = TeraServiceAccess.get_service_access_for_project(id_service=Globals.opentera_service_id,
                                                                          id_project=self.id_project)

        users = []
        for access in project_access:
            # Get all users in the related group
            if access.service_access_user_group:
                access_users = access.service_access_user_group.user_group_users
                for user in access_users:
                    if user not in users:
                        users.append(user)

        # Also appends super admins!
        # for user in TeraUser.get_superadmins():
        #     if user not in users:
        #         users.append(user)

        return users

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraSite import TeraSite
            base_project = TeraProject()
            base_project.project_name = 'Default Project #1'
            base_project.id_site = TeraSite.get_site_by_sitename('Default Site').id_site
            TeraProject.insert(base_project)

            base_project2 = TeraProject()
            base_project2.project_name = 'Default Project #2'
            base_project2.id_site = TeraSite.get_site_by_sitename('Default Site').id_site
            TeraProject.insert(base_project2)

            secret_project = TeraProject()
            secret_project.project_name = "Secret Project #1"
            secret_project.id_site = TeraSite.get_site_by_sitename('Top Secret Site').id_site
            TeraProject.insert(secret_project)

    @staticmethod
    def get_project_by_projectname(projectname):
        return TeraProject.query.filter_by(project_name=projectname).first()

    @staticmethod
    def get_project_by_id(project_id):
        return TeraProject.query.filter_by(id_project=project_id).first()

    @staticmethod
    def query_data(filter_args):
        if isinstance(filter_args, tuple):
            return TeraProject.query.filter_by(*filter_args).all()
        if isinstance(filter_args, dict):
            return TeraProject.query.filter_by(**filter_args).all()
        return None

    @classmethod
    def delete(cls, id_todel):
        super().delete(id_todel)

        # from opentera.db.models.TeraSession import TeraSession
        # TeraSession.delete_orphaned_sessions()

    @classmethod
    def insert(cls, project):
        # Creates admin and user roles for that project
        super().insert(project)

        from opentera.db.models.TeraServiceRole import TeraServiceRole
        from opentera.db.models.TeraService import TeraService
        opentera_service_id = TeraService.get_openteraserver_service().id_service

        access_role = TeraServiceRole()
        access_role.id_service = opentera_service_id
        access_role.id_project = project.id_project
        access_role.service_role_name = 'admin'
        db.session.add(access_role)

        access_role = TeraServiceRole()
        access_role.id_service = opentera_service_id
        access_role.id_project = project.id_project
        access_role.service_role_name = 'user'
        db.session.add(access_role)

        db.session.commit()



from opentera.db.Base import db, BaseModel


class TeraUserUserGroup(db.Model, BaseModel):
    __tablename__ = 't_users_users_groups'
    id_user_user_group = db.Column(db.Integer, db.Sequence('id_user_user_group_sequence'), primary_key=True,
                                   autoincrement=True)
    id_user = db.Column(db.Integer, db.ForeignKey("t_users.id_user", ondelete='cascade'), nullable=False)
    id_user_group = db.Column(db.Integer, db.ForeignKey("t_users_groups.id_user_group"),
                              nullable=False)

    user_user_group_user = db.relationship("TeraUser")
    user_user_group_user_group = db.relationship("TeraUserGroup")  # Fun variable name!

    def to_json(self, ignore_fields=[], minimal=False):
        ignore_fields.extend(['user_user_group_user', 'user_user_group_user_group'])

        if minimal:
            ignore_fields.extend([])

        rval = super().to_json(ignore_fields=ignore_fields)

        return rval

    @staticmethod
    def create_defaults(test=False):
        if test:
            from opentera.db.models.TeraUser import TeraUser
            from opentera.db.models.TeraUserGroup import TeraUserGroup
            user1 = TeraUser.get_user_by_username('siteadmin')
            user2 = TeraUser.get_user_by_username('user')
            user3 = TeraUser.get_user_by_username('user2')
            user4 = TeraUser.get_user_by_username('user3')
            group1 = TeraUserGroup.get_user_group_by_group_name("Users - Projects 1 & 2")
            group2 = TeraUserGroup.get_user_group_by_group_name("Admins - Project 1")
            group3 = TeraUserGroup.get_user_group_by_group_name("Admins - Default Site")
            group4 = TeraUserGroup.get_user_group_by_group_name("Users - Project 1")

            user_ug = TeraUserUserGroup()
            user_ug.user_user_group_user = user1
            user_ug.user_user_group_user_group = group3
            db.session.add(user_ug)

            user_ug = TeraUserUserGroup()
            user_ug.user_user_group_user = user2
            user_ug.user_user_group_user_group = group1
            db.session.add(user_ug)

            user_ug = TeraUserUserGroup()
            user_ug.user_user_group_user = user3
            user_ug.user_user_group_user_group = group4
            db.session.add(user_ug)

            user_ug = TeraUserUserGroup()
            user_ug.user_user_group_user = user3
            user_ug.user_user_group_user_group = group3
            db.session.add(user_ug)

            user_ug = TeraUserUserGroup()
            user_ug.user_user_group_user = user4
            user_ug.user_user_group_user_group = group2
            db.session.add(user_ug)

            db.session.commit()

    @staticmethod
    def get_user_user_group_by_id(user_user_group_id: int):
        return TeraUserUserGroup.query.filter_by(id_user_user_group=user_user_group_id).first()

    @staticmethod
    def query_users_for_user_group(user_group_id: int):
        return TeraUserUserGroup.query.filter_by(id_user_group=user_group_id).all()

    @staticmethod
    def query_users_groups_for_user(user_id: int):
        return TeraUserUserGroup.query.filter_by(id_user=user_id).all()

    @staticmethod
    def query_user_user_group_for_user_user_group(user_id: int, user_group_id: int):
        return TeraUserUserGroup.query.filter_by(id_user=user_id, id_user_group=user_group_id).first()

    @staticmethod
    def insert_user_user_group(id_user_group: int, id_user: int):
        new_uug = TeraUserUserGroup()
        new_uug.id_user_group = id_user_group
        new_uug.id_user = id_user

        db.session.add(new_uug)
        db.session.commit()

        return new_uug
